SystemOrganization addCategory: #'Stack-Exercise'!

Object subclass: #BaseElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!BaseElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:38'!
distanceToBase
	^ 0! !

!BaseElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:57'!
previousElement
	Error signal: 'Stack is empty'! !

!BaseElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:58'!
value 
	Error signal: 'Stack is empty'! !

Object subclass: #OOStack
	instanceVariableNames: 'elementAtTop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 5/7/2012 11:51'!
stackEmptyErrorDescription
	
	^ 'Stack is empty'! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:59'!
initialize
	elementAtTop := BaseElement new! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 02:03'!
isEmpty 
	^self size == 0! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:19'!
pop
	| poppedElement |
	
	poppedElement := elementAtTop.
	elementAtTop := elementAtTop previousElement.
	
	^poppedElement value.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 02:02'!
push: anElementValue 

	| newStackedElement |
	
	newStackedElement := StackedElement new initializeWithValue: anElementValue after: elementAtTop.
	elementAtTop := newStackedElement.! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 02:03'!
size
	^ elementAtTop distanceToBase.
	! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 02:03'!
top
	^elementAtTop value! !

Object subclass: #StackedElement
	instanceVariableNames: 'previousElement value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!StackedElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:40'!
distanceToBase
	^ 1 + previousElement distanceToBase.! !

!StackedElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:39'!
initializeWithValue: anElementValue after: anElement 
	value := anElementValue.
	previousElement := anElement.! !

!StackedElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:53'!
previousElement
	^ previousElement! !

!StackedElement methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/8/2013 01:55'!
value 
	^ value! !

TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:31'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/8/2012 08:20'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:33'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'First'.
	secondPushedObject := 'Second'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:35'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:36'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:38'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/16/2012 16:53'!
test09CanNotPopWhenThereAreNoObjectsInTheStackAndTheStackHadObjects

	| stack  |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/16/2012 16:53'!
test10CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !
